/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  KeychainHandler.m
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Thu Jan 08 2004
//
//  $Id: KeychainHandler.m,v 1.9 2007/10/18 00:02:27 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// include files
/////////////////////////////////////////////////////////////////////

#import "KeychainHandler.h"
#import <Security/Security.h>

/////////////////////////////////////////////////////////////////////
// class KeychainHandler
/////////////////////////////////////////////////////////////////////

@interface KeychainHandler ( PrivateMethods )
- (BOOL) searchKeychainForService;
- (void) dispatchError:(NSString*)errorString;
@end

static NSString* stringFromBytes( const char* bytes, UInt32 length )
{
   NSData* data = [NSData dataWithBytes:bytes length:length];
   NSString* str = [[NSString alloc] initWithData:data
                                     encoding:NSUTF8StringEncoding];
   return( [str autorelease] );
}

@implementation KeychainHandler

- (id) initWithService:(NSString*)service
       errorDelegate:(id)delegate
{
   if ( self = [super init] )
   {
      m_service = [service copy];
      m_delegate = delegate;
      if ( ! [self searchKeychainForService] )
      {
         [self release];
         return( nil );
      }
   }
   return( self );
}

- (id) init
{
   return( nil ); // not allowed
}

- (void) dealloc
{
   [m_service release];
   [m_data release];
   [super dealloc];
}

- (int) accountCount
{
   return( [m_data count] );
}

- (NSEnumerator*) accountEnumerator
{
   return( [m_data keyEnumerator] );
}

- (NSString*) passphraseForAccount:(NSString*)account
{
   return( [m_data objectForKey:account] );
}

+ (NSString*) passphraseForService:(NSString*)service
              withAccount:(NSString*)account;
{
   UInt32 passLength = 0;
   void*  passData = NULL;
   
   OSStatus status =
      SecKeychainFindGenericPassword( NULL, // default keychain search
                                      [service length],
                                      [service UTF8String],
                                      [account length],
                                      [account UTF8String],
                                      &passLength,
                                      &passData,
                                      NULL
                                    );

   if ( status != noErr || passData == NULL )
   {
      NSLog( @"Could not get passphrase for key %@ acct %@ (status %d)",
             service, account, status );
      return( nil );
   }
   
   NSString* passphrase = stringFromBytes( passData, passLength );

   SecKeychainItemFreeContent( NULL, passData );
   
   return( passphrase );
}

@end

@implementation KeychainHandler ( PrivateMethods )

- (BOOL) searchKeychainForService
{
   NSMutableDictionary* d = [[NSMutableDictionary alloc] init];
   m_data = d;
   
   // create the attribute list structure to find all generic passwords
   // with the given service name
   SecKeychainAttribute attr;
   attr.tag = kSecServiceItemAttr;
   attr.length = [m_service length];
   attr.data = (char*)[m_service UTF8String];
   SecKeychainAttributeList attrList;
   attrList.count = 1;
   attrList.attr = &attr;
   
   // use this attribute list to create a search reference
   SecKeychainSearchRef searchRef;
   OSStatus status;
   status = SecKeychainSearchCreateFromAttributes( NULL, // default keychains
                                                   kSecGenericPasswordItemClass,
                                                   &attrList,
                                                   &searchRef );
   if ( status != noErr )
   {
      [self dispatchError:@"Unable to search keychain(s) for ssh-agent keys."];
      return( NO );
   }
      
   // iterate over the resulting matches and store account/passphrase data
   SecKeychainItemRef itemRef;
   while ( ( status = SecKeychainSearchCopyNext( searchRef, &itemRef ) )
           == noErr ) 
   {
      // the documentation here sucks, but if I don't include an entry
      // in AttributeInfo for the tag I want, I get nothing; I have no
      // idea what "format" means, but setting to zero seems to work!
      SecKeychainAttributeInfo attrInfo;
      UInt32 infoTag = kSecAccountItemAttr;
      UInt32 infoFormat = 0;
      attrInfo.count = 1;
      attrInfo.tag = &infoTag;
      attrInfo.format = &infoFormat;
      
      // the AttributeList is alleged to be an input, as well as output,
      // parameter, but replicates the (undocumented) data in
      // AttributeInfo; we treat it as output only that works fine!
      SecKeychainAttributeList* outAttrList = NULL;
      
      // finally get the damn attributes and data (passphrase)
      UInt32 outLength;
      void* outData;
      status = SecKeychainItemCopyAttributesAndData( itemRef,
                                                     &attrInfo,
                                                     NULL,
                                                     &outAttrList,
                                                     &outLength,
                                                     &outData );
      if ( status != noErr )
      {
         CFRelease( itemRef );
         break;
      }
      
      // get the account name (from attributes)
      NSString* account = nil;
      UInt32 numAttrs = outAttrList->count;
      SecKeychainAttribute* nextAttr = outAttrList->attr;
      while ( numAttrs-- != 0 )
      {
         if ( nextAttr->tag == kSecAccountItemAttr )
         {
            account = stringFromBytes( outAttrList->attr->data,
                                       outAttrList->attr->length );
         }
         nextAttr++;
      }
      
      // get the passphrase (out data)
      NSString* passphrase = stringFromBytes( outData, outLength );
      
      // if we got both pieces, set in dictionary
      if ( account != nil && passphrase != nil )
         [d setObject:passphrase forKey:account];
      
      // clean up
      SecKeychainItemFreeAttributesAndData( outAttrList, outData );
      CFRelease( itemRef );
   }
   if ( status != noErr && status != errSecItemNotFound ) // end of list
   {
      // check for some likely error cases, although the only one I can
      // get to occur is the undocumented one (-128)!
      NSString* errorString = nil;
      switch ( status )
      {
      case errSecAuthFailed:
      case userCanceledErr: // -128
         errorString = @"SSHChain was not allowed to access the keychain.";
         break;
      case errSecNoSuchAttr:
         errorString = @"Account name was not specified in keychain "
                       @"for ssh-agent.";
         break;
      case errSecNoDefaultKeychain:
         errorString = @"No default keychain was found.";
         break;
      case errSecDataNotAvailable:
         errorString = @"The passphrase is missing from the keychain.";
         break;
      default:
         errorString = [NSString stringWithFormat:
                       @"Unable to access keychain data (%d).", status];
      }
      [self dispatchError:errorString];
   }
   
   // clean up the search criteria
   CFRelease( searchRef );
   
   return( status == errSecItemNotFound ); 
}

- (void) dispatchError:(NSString*)errorString
{
   if ( m_delegate == nil )
      return;
   if ( [m_delegate respondsToSelector:@selector( dispatchKeychainError: )] )
   {
      // let our cleanup code run before actually dispatching the
      // error, by doing a delayed perform
      [m_delegate performSelector:@selector( dispatchKeychainError: )
                  withObject:errorString
                  afterDelay:0.0];
   }
}

@end

