/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  PrefController.h
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Thu Jan 13 2004
//
//  $Id: PrefController.h,v 1.11 2007/10/02 04:50:18 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>

/////////////////////////////////////////////////////////////////////
// class PrefController
/////////////////////////////////////////////////////////////////////

extern NSString* SavePreferencesNotification;
extern NSString* AgentRestartRequestNotification;
extern NSString* SocketPathChangedNotification;
extern NSString* AgentEnvFileChangedNotification;

/*!
 * @typedef PrefControllerAgentShell
 *
 * An enumeration used by PrefController to denote the user's
 * preferred syntax for the environment file. For implementation
 * purposes, the values of this enum match the NSCell tags of
 * the radio buttons in the nib file.
 *
 * @constant AGENT_SHELL_CSH
 *    use csh-compatible syntax for environment file
 * @constant AGENT_SHELL_SH
 *    use sh-compatible syntax for environment file
 */
typedef enum _PrefControllerAgentShell
{
   AGENT_SHELL_CSH = 0,
   AGENT_SHELL_SH = 1
} PrefControllerAgentShell;

/*!
 * @class PrefController
 *
 * The PrefController class manages preference settings and
 * the preferences dialog. The singleton object can retrieved
 * to get the current preference settings (the associated nib
 * will be loaded only if the NSWindowController window method
 * is called).
 */
@interface PrefController : NSWindowController
{

   IBOutlet NSMatrix*    m_sockPathRadio;
   IBOutlet NSTextField* m_sockPathTextField;
   IBOutlet NSButton*    m_sockPathButton;
   IBOutlet NSButton*    m_createEnvCheckbox;
   IBOutlet NSTextField* m_envPathTextField;
   IBOutlet NSButton*    m_envPathButton;
   IBOutlet NSMatrix*    m_envFormatRadio;
   IBOutlet NSButton*    m_loginItemCheckbox;
   
   IBOutlet NSWindow*    m_aboutWindow;
   IBOutlet NSTextField* m_aboutVersionTextField;
   IBOutlet NSTextField* m_aboutCopyTextField;
   IBOutlet NSButton*    m_aboutURLButton;
}

/*!
 * @method sharedPrefController
 *
 * Return the single shared PrefController object for app.
 *
 * @result
 *    the shared object
 */
+ (PrefController*) sharedPrefController;

/*!
 * @method upgradePreferences
 *
 * Upgrade any invalidated preferences from a previous
 * app version. After upgrading to the current version,
 * future calls will be a no-op.
 */
- (void) upgradePreferences;

/*!
 * @method explictSocketPath
 *
 * Get the user-specified socket path, if any. If no socket
 * path is specified, nil will be returned (indicating that
 * a default socket should be assigned).
 *
 * @result
 *    socket path, or nil if none set
 */
- (NSString*) explicitSocketPath;

/*!
 * @method setExplicitSocketPath:
 *
 * Set the user-specified socket path as given. To clear the
 * socket path (and let the default be used), pass nil.
 * 
 * @param path
 *    the new socket path, or nil to clear
 */
- (void) setExplicitSocketPath:(NSString*)path;

/*!
 * @method agentEnvironmentFile
 *
 * Get the user-specified path for the agent environment file,
 * or nil if the file is not to be written.
 *
 * @result
 *    file path, or nil if no file
 */
- (NSString*) agentEnvironmentFile;

/*!
 * @method setAgentEnvironmentFile:
 *
 * Set the agent environment file path to write. To inhibit the
 * creation of the environment file, pass nil.
 * 
 * @param path
 *    new environment file path, or nil to clear
 */
- (void) setAgentEnvironmentFile:(NSString*)path;

/*!
 * @method agentEnvironmentShell
 *
 * Get the user-specified format for the agent environment file.
 *
 * @result
 *    file format
 */
- (PrefControllerAgentShell) agentEnvironmentShell;

/*!
 * @method setAgentEnvironmentShell:
 *
 * Set the agent environment file format to write.
 * 
 * @param path
 *    new environment file format
 */
- (void) setAgentEnvironmentShell:(PrefControllerAgentShell)shell;

- (IBAction) choosePath:(id)sender;
- (IBAction) changeSockPath:(id)sender;
- (IBAction) changeEnvFile:(id)sender;
- (IBAction) quitProgram:(id)sender;
- (IBAction) showAboutPanel:(id)sender;
- (IBAction) dismissAboutPanel:(id)sender;
- (IBAction) savePrefs:(id)sender;
- (IBAction) discardPrefs:(id)sender;
- (IBAction) showWebPage:(id)sender;

@end
