/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  SSHAgent.h
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Tue Jan 06 2004
//
//  $Id: SSHAgent.h,v 1.6 2004/04/04 00:16:21 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

/////////////////////////////////////////////////////////////////////
// class SSHAgent
/////////////////////////////////////////////////////////////////////

/*!
 * @class SSHAgent
 *
 * The SSHAgent class creates and interacts with an instance of
 * ssh-agent. The launch method is used to start the agent, and
 * then addKey:usingPassword: can be used to add any number of
 * keys to the agent thus started. Before termination, shutdown
 * should be invoked to terminate the forked ssh-agent process.
 */
@interface SSHAgent : NSObject
{
   NSString*     m_reqSock;
   NSString*     m_authSocket;
   int           m_agentPID;
   NSDictionary* m_agentEnv;
   id            m_delegate;
}

/*!
 * @method init
 *
 * Creates a new ssh-agent handler with default socket path.
 *
 * @result
 *    initialized SSHAgent object
 */
- (id) init;
 
/*!
 * @method initWithSocketName:
 *
 * The designated initializer. Creates a new ssh-agent handler
 * with (optionally) a preferred socket path. If sockName is 
 * non-nil, it will be used for ssh-agent -a. Otherwise, 
 * ssh-agent will be allowed to choose a default socket. Giving
 * an explicit socket makes for a predictable SSH_AUTH_SOCK that
 * can be set statically.
 *
 * Errors encountered in SSHAgent operations are dispatched
 * through the given delegate, if any. See the SSHAgent_ErrorDelegate
 * category for the informal protocol to be implemented by the
 * delegate object.
 *
 * @param sockName
 *    the preferred socket path, or nil to use default
 * @param delegate
 *    delegate for dispatching errors, or nil if none
 * @result
 *    initialized SSHAgent object
 */
- (id) initWithSocketName:(NSString*)sockName
       errorDelegate:(id)delegate;

/*!
 * @method launch
 *
 * Launch the ssh-agent process. Returns NO if for any reason
 * the agent could not be successfully launched; use errorString
 * to retrieve a user-appropriate error message.
 *
 * @result
 *    YES if launched okay, else NO
 */
- (BOOL) launch;

/*!
 * @method shutdown
 *
 * Shut down the ssh-agent process. Intended to be called prior
 * to terminating the SSHAgent client, so that the agent is not
 * left running (with keys installed).
 */
- (void) shutdown;

/*!
 * @method isValid
 *
 * Determine if ssh-agent was launched successfully, such that
 * keys can be added and/or an external environment initialized.
 *
 * @result
 *    YES if agent is running, else NO
 */
- (BOOL) isValid;
 
/*!
 * @method authSocket
 *
 * Get path for UNIX domain socket to ssh-agent. Valid only if
 * the agent has been launched. Can be used to set an external
 * environment's SSH_AUTH_SOCK variable. Of course, if a socket
 * path was given on construction, it should match this one.
 *
 * @result
 *    the socket path, or nil if agent not running
 */
- (NSString*) authSocket;
 
/*!
 * @method agentPID
 *
 * Get PID for ssh-agent. Can be used to set an external
 * environment's SSH_AGENT_PID variable.
 *
 * @result
 *    the agent PID, or -1 if agent not running
 */
- (int) agentPID;
 
/*!
 * @method addKey:usingPassword:
 *
 * Add the given key file to the running agent, using the given
 * password to unlock it. If key is added successfully, returns
 * YES. If there is any problem adding or unlocking the key, NO
 * is returned; use errorString to retrieve a user-appropriate
 * error message.
 *
 * @param keyFile
 *    absolute path to key file to be added
 * @param password
 *    password to use to unlock keyFile
 * @result
 *    YES if key added okay, else NO
 */
- (BOOL) addKey:(NSString*)keyFile
         usingPassword:(NSString*)password;

@end

/////////////////////////////////////////////////////////////////////
#pragma mark ERROR DELEGATE INFORMAL PROTOCOL
/////////////////////////////////////////////////////////////////////

/*!
 * @category NSObject ( SSHAgent_ErrorDelegate )
 *
 * Informal protocol used for SSHAgent's error delegate.
 */
@interface NSObject ( SSHAgent_ErrorDelegate )

/*!
 * @method dispatchAgentError:
 *
 * Error messages are dispatched through this delegate method.
 *
 * @param errorString
 *    a user-appropriate string describing the error encountered
 */   
- (void) dispatchAgentError:(NSString*)errorString;

@end


