/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  SSHAgent.m
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Tue Jan 06 2004
//
//  $Id: SSHAgent.m,v 1.12 2007/10/18 00:02:27 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// include files
/////////////////////////////////////////////////////////////////////

#import "SSHAgent.h"
#include <signal.h>
#include <string.h> // strerror()

/////////////////////////////////////////////////////////////////////
// constants
/////////////////////////////////////////////////////////////////////

#define AUTH_ENV @"SSH_AUTH_SOCK"
#define PID_ENV  @"SSH_AGENT_PID"

/////////////////////////////////////////////////////////////////////
// class SSHAgent
/////////////////////////////////////////////////////////////////////

@interface SSHAgent ( PrivateMethods )
- (BOOL) launchAgentTask;
- (BOOL) analyzeAgentOutput:(NSString*)outString;
- (BOOL) runAddForKey:(NSString*)keyPath
         usingPassword:(NSString*)password;
- (NSString*) commandLineForTask:(NSTask*)task;
- (void) dispatchError:(NSString*)errorString;
@end

@implementation SSHAgent

- (id) initWithSocketName:(NSString*)sockName
       errorDelegate:(id)delegate
{
   if ( self = [super init] )
   {
      m_reqSock = [sockName copy];
      m_agentPID = -1;
      m_authSocket = nil;
      m_agentEnv = nil;
      m_delegate = delegate;
   }
   return( self );
}

- (id) init
{
   return( [self initWithSocketName:nil errorDelegate:nil] );
}

- (void) dealloc
{
   [m_authSocket release];
   [m_agentEnv release];
   [super dealloc];
}

- (NSString*) description
{
   NSString* d = @"SSHAgent:\n";
   d = [d stringByAppendingFormat:@"   PID %d\n", m_agentPID];
   d = [d stringByAppendingFormat:@"   auth socket %@\n", m_authSocket];
   return( d );
}
 
- (BOOL) launch
{
   return( [self launchAgentTask] );
}

- (void) shutdown
{
   if ( m_agentPID != -1 )
   {
      int rc = kill( m_agentPID, SIGTERM );
      if ( rc == -1 )
      {
         [self dispatchError:[NSString stringWithFormat:
                  @"Unable to terminate ssh-agent process with ID %d (%d: %s).",
                  m_agentPID,
                  errno,
                  strerror( errno )]];
      }
   }
}
 
- (NSString*) authSocket
{
   return( m_authSocket );
}

- (BOOL) isValid
{
   return( m_agentPID != -1 && m_authSocket != nil );
}
 
- (int) agentPID
{
   return( m_agentPID );
}
 
- (BOOL) addKey:(NSString*)keyFile
         usingPassword:(NSString*)password
{
   return( [self runAddForKey:keyFile usingPassword:password] );
}

@end

@implementation SSHAgent ( PrivateMethods )

- (BOOL) launchAgentTask
{
   // set up task and get pipe for stdout
   NSTask* task = [[NSTask alloc] init];
   NSPipe* outPipe = [NSPipe pipe];
   NSFileHandle* readHandle = [outPipe fileHandleForReading];
   [task setStandardOutput:outPipe];

   // connect stderr so we can get diagnostics if we fail
   NSPipe* errPipe = [NSPipe pipe];
   NSFileHandle* errHandle = [errPipe fileHandleForReading];
   [task setStandardError:errPipe];
   
   // initialize task command
   [task setLaunchPath:@"/usr/bin/ssh-agent"];
   NSMutableArray* args = [NSMutableArray array];
   [args addObject:@"-c"];
   if ( m_reqSock != nil )
   {
      [args addObject:@"-a"];
      [args addObject:m_reqSock];
   }
   [task setArguments:args];
   
   // launch it, get the pid and wait for termination
   [task launch];
   [task waitUntilExit]; // block until done
   
   // get the stdout information (even if it failed, we want this)
   NSData* outData = [readHandle readDataToEndOfFile];
   NSString* outString = [[NSString alloc] initWithData:outData
                                           encoding:NSASCIIStringEncoding];
   [outString autorelease];

   // check for non-zero from ssh-agent and dispatch error message
   BOOL good = ( [task terminationStatus] == 0 );
   if ( ! good )
   {
      // get text from stderr and append to a generic message
      NSData* errData = [errHandle readDataToEndOfFile];
      NSString* errString = [[NSString alloc] initWithData:errData
                                              encoding:NSASCIIStringEncoding];
      [self dispatchError:[NSString stringWithFormat:
               @"Attempt to launch \"%@\" failed with exit code %d:\n%@",
               [self commandLineForTask:task], 
               [task terminationStatus],
               errString]];
      [errString release];
   }

   // if it worked, extract the auth socket
   else
   {
      if ( ! [self analyzeAgentOutput:outString] )
      {
         good = NO;
         [self dispatchError:[NSString stringWithFormat:
                  @"Unexpected output \"%@\" from command \"%@\".",
                  outString, [self commandLineForTask:task]]];
      }
   }
   
   // if we couldn't launch or extract socket, bail
   if ( ! good )
   {
      m_agentPID = -1;
      [m_authSocket release];
      m_authSocket = nil;
   }
   
   // clean and return
   [task release];
   return( good );
}

- (BOOL) analyzeAgentOutput:(NSString*)outString
{
   NSScanner* scanner = [NSScanner scannerWithString:outString];
   NSCharacterSet* skipSet = 
      [NSCharacterSet characterSetWithCharactersInString:@" \t\r\n;"];
   [scanner setCharactersToBeSkipped:skipSet];
   
   // scan the socket information
   if ( ! [scanner scanUpToString:AUTH_ENV intoString:nil]
        || ! [scanner scanString:AUTH_ENV intoString:nil] )
      return( NO );
   [scanner scanCharactersFromSet:skipSet intoString:nil];
   NSString* sockName = nil;
   if ( ! [scanner scanUpToCharactersFromSet:skipSet
                   intoString:&sockName] )
      return( NO );
   m_authSocket = [sockName copy];

   // scan the (forked) agent PID
   if ( ! [scanner scanUpToString:PID_ENV intoString:nil]
        || ! [scanner scanString:PID_ENV intoString:nil] )
      return( NO );
   [scanner scanCharactersFromSet:skipSet intoString:nil];
   int pid = -1;
   if ( ! [scanner scanInt:&pid] )
      return( NO );
   m_agentPID = pid;
   
   return( YES );
}

- (BOOL) runAddForKey:(NSString*)keyPath
         usingPassword:(NSString*)password
{
   // bail if we couldn't or didn't start agent
   if ( ! [self isValid] )
      return( NO );
   
   // init the environment if we haven't done it yet
   if( m_agentEnv == nil )
   {
      m_agentEnv = [[NSDictionary dictionaryWithObject:m_authSocket
                                  forKey:AUTH_ENV] retain];
   }

   // set up task and get pipe for stdin; stdout not particularly
   // interesting, so send it to /dev/null
   NSTask* task = [[NSTask alloc] init];
   NSPipe* inPipe = [NSPipe pipe];
   NSFileHandle* writeHandle = [inPipe fileHandleForWriting];
   [task setStandardInput:inPipe];
   [task setStandardOutput:[NSFileHandle fileHandleWithNullDevice]];

   // connect stderr so we can get diagnostics if we fail
   NSPipe* errPipe = [NSPipe pipe];
   NSFileHandle* errHandle = [errPipe fileHandleForReading];
   [task setStandardError:errPipe];
   
   // initialize task command
   [task setLaunchPath:@"/usr/bin/ssh-add"];
   [task setArguments:[NSArray arrayWithObject:keyPath]];
   [task setEnvironment:m_agentEnv];
   
   // build the password data; cheat and send an extra newline,
   // so that if the password is bad, it will bail
   NSString* response = [password stringByAppendingString:@"\n\n"];
   NSData* inData =
      [response dataUsingEncoding:[NSString defaultCStringEncoding]];
   
   // launch it, stuff in the password and wait for termination
   [task launch];
   [writeHandle writeData:inData];
   [task waitUntilExit]; // block until done
   
   // return exit status
   int good = ( [task terminationStatus] == 0 );
   if ( ! good )
   {
      // get text from stderr and append to a generic message; but if
      // we get a bad passphrase message (this does goes to stderr),
      // replace it with something more useful
      NSData* errData = [errHandle readDataToEndOfFile];
      NSString* errString = [[NSString alloc] initWithData:errData
                                              encoding:NSASCIIStringEncoding];
      if ( [errString rangeOfString:@"Bad passphrase"].location != NSNotFound )
      {
         [errString release];
         errString = @"Incorrect passphrase for key. Check password "
                     @"entered in Keychain.";
      }
      [self dispatchError:[NSString stringWithFormat:
               @"Attempt to add key \"%@\" using ssh-add failed "
               @"with exit code %d:\n%@",
               keyPath, [task terminationStatus], errString]];
      [errString release];
   }
   [task release];
   return( good );
}

- (NSString*) commandLineForTask:(NSTask*)task
{
   NSString* cmd = [task launchPath];
   NSEnumerator* e = [[task arguments] objectEnumerator];
   NSString* arg;
   while ( arg = [e nextObject] )
      cmd = [cmd stringByAppendingFormat:@" %@", arg];

   return( cmd );
}

- (void) dispatchError:(NSString*)errorString
{
   if ( m_delegate == nil )
      return;
   if ( [m_delegate respondsToSelector:@selector( dispatchAgentError: )] )
   {
      // let our cleanup code run before actually dispatching the
      // error, by doing a delayed perform
      [m_delegate performSelector:@selector( dispatchAgentError: )
                  withObject:errorString
                  afterDelay:0.0];
   }
}

@end
